require "collision"

if not table.unpack then
    table.unpack = unpack
end

Ball = {}
Ball.__index = Ball

function Ball.new(x, y)
	local self = {}
	self.position = vec2.new(x, y)
	self.velocity = vec2.new(0, 1)
	self.radius = 5
	return setmetatable(self, Ball)

end

function Ball:draw() 
	love.graphics.circle("fill",self.position.x, self.position.y, self.radius)
end


function Ball:update(dt)
	self.velocity.y = self.velocity.y + 9.81/40 -- apply gravity factor

	local collisions = {}


	pos = self.position

	for i=1, #terrain do
		local o = terrain[i]
		normal = checkCollision(self,o) -- Check collision between a circle and a SDF
		if normal then
			table.insert(collisions, normal)
		end
	end

	if #collisions > 0 then

		-- In case of multiple collisions
		local combinedNormal = vec2.new(0, 0)
		for _, collision in ipairs(collisions) do
			combinedNormal = combinedNormal + collision
		end

		-- Little physics stuff
		combinedNormal = combinedNormal:normalized()
		local velocityPerpendicular = self.velocity:projectedAlong(combinedNormal)
		local velocityParallel = self.velocity - velocityPerpendicular
		velocityPerpendicular = -velocityPerpendicular * 0.13 
		self.velocity = velocityPerpendicular + velocityParallel
	end

	self.position = self.position + self.velocity * dt * 3
	if self.position.y > 800 then
		self.position = vec2.new(40, 20)
		self.velocity = vec2.new(0, 1)
	end
end