require "math"
local sqrt = math.sqrt


 -- vec2 metatable:
vec2 = {}
vec2.__index = vec2

-- vec2 constructor:
function vec2.new(x, y)
  local v = {x = x or 0, y = y or 0}
  setmetatable(v, vec2)
  return v
end

-- vec2 addition:
function vec2.__add(a, b)
  return vec2.new(a.x + b.x, a.y + b.y)
end

-- vec2 subtraction:
function vec2.__sub(a, b)
  return vec2.new(a.x - b.x, a.y - b.y)
end

-- multiplication of a vec2 by a scalar:
function vec2.__mul(a, b)
  if type(a) == "number" then
    return vec2.new(b.x * a, b.y * a)
  elseif type(b) == "number" then
    return vec2.new(a.x * b, a.y * b)
  else
    error("Can only multiply vec2 by scalar.")
  end
end

-- dividing a vec2 by a scalar:
function vec2.__div(a, b)
   if type(b) == "number" then
      return vec2.new(a.x / b, a.y / b)
   else
      error("Invalid argument types for vec2 division.")
   end
end

-- vec2 equivalence comparison:
function vec2.__eq(a, b)
  return a.x == b.x and a.y == b.y
end

-- vec2 not equivalence comparison:
function vec2.__ne(a, b)
  return not vec2.__eq(a, b)
end

-- unary negation operator:
function vec2.__unm(a)
  return vec2.new(-a.x, -a.y)
end

-- vec2 < comparison:
function vec2.__lt(a, b)
   return a.x < b.x and a.y < b.y
end

-- vec2 <= comparison:
function vec2.__le(a, b)
   return a.x <= b.x and a.y <= b.y
end

-- vec2 value string output:
function vec2.__tostring(v)
   return "(" .. v.x .. ", " .. v.y .. ")"
end

function vec2:length()
  return sqrt(self:dot(self))
end

function vec2.dot(a, b)
  return a.x * b.x + a.y * b.y
end

function vec2:normalized()
  local r = self:length()
  if r > 0 then
    local div = 1 / r
    return vec2.new(self.x * div, self.y * div)
  else
    -- no particular direction; pick one at random!
    local a = random() * twopi
    return vec2.new(cos(a), sin(a))
  end
  return self
end


function vec2:projectedAlong(y)
  local projection = y * (self:dot(y) / y:dot(y))
  return vec2.new(projection.x, projection.y)
end

